//==============================================================================
// IOPINS - Handle some of the IO functions of the BAP
//==============================================================================
#include "BapLib.h"
#include <stdarg.h>

#ifdef __cplusplus
extern "C" {
#endif

#define _LOW_ 						0
#define _HIGH_						1

#define _BA_PIN_MODE_IO_			0
#define _BA_PIN_MODE_FUNCTION_		1

#define _BA_PIN_MODE_INPUT_			0
#define _BA_PIN_MODE_OUTPUT_		1

#define _BA_ADC_MODE_SINGLE_		0
#define _BA_ADC_MODE_SCAN_			1

#define _BA_ADC_CLOCK_134_			0
#define _BA_ADC_CLOCK_70_			1

enum _PCR_SHADOW_REGS_ {_PCR1_SHADOW_, _PCR2_SHADOW_, _PCR3_SHADOW_, _PCR4_SHADOW_, cPCR5_SHADOW, _PCR6_SHADOW_, _PCR7_SHADOW_, _PCR8_SHADOW_, _PCR_SHADOW_COUNT_};
u8 bap_bpcrshadowregs [_PCR_SHADOW_COUNT_];

const PORTITEM	g_PortTable[] =
{
#ifdef _BAP28_
	{&IO.PDR5.BYTE, 0x1, (PUR_5 | PMR_5)},	// P0
	{&IO.PDR5.BYTE, 0x2, (PUR_5 | PMR_5)},	// P1
	{&IO.PDR5.BYTE, 0x4, (PUR_5 | PMR_5)},	// P2
	{&IO.PDR5.BYTE, 0x8, (PUR_5 | PMR_5)},	// P3
	{&IO.PDR5.BYTE, 0x10, (PUR_5 | PMR_5)},	// P4
	{&IO.PDR5.BYTE, 0x20, (PUR_5 | PMR_5)},	// P5
	{&IO.PDR5.BYTE, 0x40, 0},	// P6
	{&IO.PDR5.BYTE, 0x80, 0},	// P7
	{&IO.PDR8.BYTE, 0x1, 0},	// P8 - Does support pull-up but on P15?  so for now say no
	{&IO.PDR8.BYTE, 0x2, 0},	// P9
	{&IO.PDR8.BYTE, 0x4, 0},	// P10
	{&IO.PDR8.BYTE, 0x8, 0},	// P11
	{&IO.PDR8.BYTE, 0x10, 0},	// P12 - Does support pull-up on P10; so far now say no
	{&IO.PDR8.BYTE, 0x20, 0},	// P13
#if 1
	{&IO.PDR8.BYTE, 0x40, 0},	// P14
	{&IO.PDR8.BYTE, 0x80, 0},	// P15
#else
	{&IO.PDR2.BYTE, 0x02, 0},	// P14
	{&IO.PDR2.BYTE, 0x04, 0},	// P15
#endif	
	{&IO.PDR1.BYTE, 0x1, (PUR_1 | PMR_1)},	// P16
	{&IO.PDR1.BYTE, 0x2, (PUR_1 | PMR_1)},	// P17
	{&IO.PDR1.BYTE, 0x40, (PUR_1 | PMR_1)},	// P18
	{&IO.PDR1.BYTE, 0x80, (PUR_1 | PMR_1)},	// P19
	{0, 0, 0},	//20
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},//30
	{0, 0, 0},
	{&IO.PDR1.BYTE, 0x10, 0},	// S_IN
	{&IO.PDR1.BYTE, 0x10, 0}	// S_OUT
#endif /* _BAP28_ */

// OK lets define Bap40 pins as well
#ifdef _BAP40_
	{&IO.PDR5.BYTE, 0x1, (PUR_5 | PMR_5)},	// P0
	{&IO.PDR5.BYTE, 0x2, (PUR_5 | PMR_5)},	// P1
	{&IO.PDR5.BYTE, 0x4, (PUR_5 | PMR_5)},	// P2
	{&IO.PDR5.BYTE, 0x8, (PUR_5 | PMR_5)},	// P3
	{&IO.PDR5.BYTE, 0x10, (PUR_5 | PMR_5)},	// P4
	{&IO.PDR5.BYTE, 0x20, (PUR_5 | PMR_5)},	// P5
	{&IO.PDR5.BYTE, 0x40, 0},	// P6
	{&IO.PDR5.BYTE, 0x80, 0},	// P7
	{&IO.PDR6.BYTE, 0x1, 0},	// P8
	{&IO.PDR6.BYTE, 0x2, 0},	// P9
	{&IO.PDR6.BYTE, 0x4, 0},	// P10
	{&IO.PDR6.BYTE, 0x8, 0},	// P11
	{&IO.PDR6.BYTE, 0x10, 0},	// P12
	{&IO.PDR6.BYTE, 0x20, 0},	// P13
	{&IO.PDR6.BYTE, 0x40, 0},	// P14
	{&IO.PDR6.BYTE, 0x80, 0},	// P15
	{&IO.PDR3.BYTE, 0x1,  0},	// P16
	{&IO.PDR3.BYTE, 0x2, 0},	// P17
	{&IO.PDR3.BYTE, 0x4, 0},	// P18
	{&IO.PDR3.BYTE, 0x8,  0},	// P19
	{&IO.PDR3.BYTE, 0x10, 0},	// P20
	{&IO.PDR3.BYTE, 0x20, 0},	// P21
	{&IO.PDR3.BYTE, 0x40, 0},	// P22
	{&IO.PDR3.BYTE, 0x80, 0},	// P23
	{&IO.PDR1.BYTE, 0x1, (PUR_1 | PMR_1)},	// P24
	{&IO.PDR1.BYTE, 0x2, (PUR_1 )},		// P25 - PMR does not match
	{&IO.PDR1.BYTE, 0x4, (PUR_1 | PMR_1)},	// P26
	{&IO.PDR2.BYTE, 0x8, (PUR_20)},	// P27
	{&IO.PDR1.BYTE, 0x10, (PUR_1 | PMR_1)},	// P28
	{&IO.PDR1.BYTE, 0x20, (PUR_1 | PMR_1)},	// P29
	{&IO.PDR1.BYTE, 0x40, (PUR_1 | PMR_1)},	// P30
	{&IO.PDR1.BYTE, 0x80, (PUR_1 | PMR_1)},	// P31
	{&IO.PDR2.BYTE, 0x2, 0},	// S_IN (P21)
	{&IO.PDR2.BYTE, 0x4, 0}	// S_OUT TXD(P22)

#endif
	
};	

//Pro 28 H8/3664F Pro 28 H8/3664F
//P0 P50/WKP0 PB0/AN0 P8 P80/FTCI P15/IRQ1
//P1 P51/WKP1 PB1/AN1 P9 P81/FTIOA P74/TMRIV
//P2 P52/WKP2 PB2/AN2 P10 P82/FTIOB
//P3 P53/WKP3 PB3/AN3 P11 P83/FTIOC
//P4 P54/WKP4 P12 P84/FTIOD P10/TMOW
//P5 P55/WKP5/ADTRG P13 P85 P20/SCK3
//P6 P56/SDA P14 P86 P21/RXD
//P7 P57/SCL P15 P87 P22/TXD
//P16 P11 PB7/AN7 P18 P16/IRQ2 PB5/AN5
//P17 P12 PB6/AN6 P19 P17/IRQ3 PB4/AN4



//=============================================================================
// bap_initialize
//=============================================================================
void bap_initialize (void)
{
	// disable IRQs
	IO.PMR1.BIT.IRQ0 = _BA_PIN_MODE_IO_;
	IO.PMR1.BIT.IRQ1 = _BA_PIN_MODE_IO_;
	IO.PMR1.BIT.IRQ2 = _BA_PIN_MODE_IO_;
	IO.PMR1.BIT.IRQ3 = _BA_PIN_MODE_IO_;
	// disable interrupts
	IENR1.BIT.IEN0 = _BA_PIN_MODE_IO_;
	IENR1.BIT.IEN1 = _BA_PIN_MODE_IO_;
	IENR1.BIT.IEN2 = _BA_PIN_MODE_IO_;
	IENR1.BIT.IEN3 = _BA_PIN_MODE_IO_;
	// set PORT5 wakeup pins as IO
	IO.PMR5.BIT.WKP0 = _BA_PIN_MODE_IO_;
	IO.PMR5.BIT.WKP1 = _BA_PIN_MODE_IO_;
	IO.PMR5.BIT.WKP2 = _BA_PIN_MODE_IO_;
	IO.PMR5.BIT.WKP3 = _BA_PIN_MODE_IO_;
	IO.PMR5.BIT.WKP4 = _BA_PIN_MODE_IO_;
	IO.PMR5.BIT.WKP5 = _BA_PIN_MODE_IO_;
	// set TMOW as IO pin
	IO.PMR1.BIT.TMOW = _BA_PIN_MODE_IO_;
	// set TXD as IO pin
	IO.PMR1.BIT.TXD = _BA_PIN_MODE_IO_;
	// set RXD as IO pin
	SCI3.SCR3.BIT.RE = _BA_PIN_MODE_IO_;
	SCI3.SSR.BYTE = 0;			// clear out any status

	// set SCK3 as IO pin
	SCI3.SCR3.BIT.CKE = _BA_PIN_MODE_IO_;
	SCI3.SMR.BIT.COM = _BA_PIN_MODE_IO_;

	SCI3.SCR3.BIT.RIE = _FALSE_;		// disable interrupts when we receive a character or error
	SCI3.SCR3.BIT.TIE = _FALSE_;		// disable interrupts when ready for next byte to transmit
	SCI3.SCR3.BIT.TEIE = _FALSE_;		// For now don't need to enable interrupt when transmit complete...


	IIC2.ICIER.BYTE = 0;				// Make sure I2C is not going to generate any interrupts.
	IIC2.ICCR1.BIT.ICE = 0;				// Likewise it is disabled...

	// set PORT pins as input
	bap_bpcrshadowregs [_PCR1_SHADOW_] = 0;
	bap_bpcrshadowregs [_PCR2_SHADOW_] = 0;
	bap_bpcrshadowregs [cPCR5_SHADOW] = 0;
	bap_bpcrshadowregs [_PCR7_SHADOW_] = 0;
	bap_bpcrshadowregs [_PCR8_SHADOW_] = 0;
	// set PORT pins from shadow registers
	IO.PCR1 = bap_bpcrshadowregs [_PCR1_SHADOW_];
	IO.PCR2 = bap_bpcrshadowregs [_PCR2_SHADOW_];
	IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
	IO.PCR7 = bap_bpcrshadowregs [_PCR7_SHADOW_];
	IO.PCR8 = bap_bpcrshadowregs [_PCR8_SHADOW_];
	// disable input pullups
	IO.PUCR1.BYTE = 0x00;
	IO.PUCR5.BYTE = 0x00;

	#ifdef _BAP28_
	// set RXD as IO pin
	SCI3.SCR3.BIT.RE = _BA_PIN_MODE_IO_;
	// disable FTIO
	TW.TIOR0.BIT.IOA = 0x00;
	TW.TIOR0.BIT.IOB = 0x00;
	TW.TIOR1.BIT.IOC = 0x00;
	TW.TIOR1.BIT.IOD = 0x00;
	// disable analog to digital
	AD.ADCSR.BIT.ADST = 0;
	// disable timer interrupts
	IENR1.BIT.IENTA = 0; // timera
	// disable timer counting
	TW.TMRW.BIT.CTS = 0; // timerw
	#endif
}



// pause 0.5 microseconds
//=============================================================================
// pauseus (u32 lpause)
//=============================================================================
void pauseus (u32 _lpause)
{
	u32 lpause;
	for (lpause=0; lpause<_lpause; lpause++);
}

//=============================================================================
// pause (u32 lpause) - IN MILLISECONDS
//=============================================================================
void pause (u32 _lpause)
{
	pauseus ((_lpause*2000));
}



//=============================================================================
// input (u8 _bpin)
//=============================================================================
void input (u8 _bpin, bool fPullup)
{
	// Lets use our table to do the work for us...
	if (_bpin < (sizeof(g_PortTable)/sizeof(PORTITEM)))	{
		// Ok lets get the port table item...
		PORTITEM pi = g_PortTable[_bpin];
		
		// Make sure it is a valid one...
		if (pi.pPDR){
			// See if there is a PMR register associated with this pin.
			if ((pi.bPU_PM) & 0xf) {
				volatile register unsigned char *pPMR = &IO.PMR1.BYTE + (pi.bPU_PM & 0xf) - 1;	// get the right PMR register
				*pPMR &= ~(pi.bMask);
			}
			
			// Next lets update PDR/PCR...
			u8 bIOPortIndex = (pi.pPDR - &IO.PDR1.BYTE);
			volatile unsigned char *pPCR = &IO.PCR1 + bIOPortIndex;
			bap_bpcrshadowregs[bIOPortIndex] &= ~(pi.bMask);				// Update shadow mask to say that the pin is now an input pin
			*pPCR = bap_bpcrshadowregs[bIOPortIndex];
			
			// Now lets see if we have a PUCR register (pull-up) to update for this pin
			if ((pi.bPU_PM) >> 4) {
				volatile register unsigned char *pPUCR = &IO.PUCR1.BYTE + (pi.bPU_PM >> 4) - 1;	// get the right PUCR register
				if (fPullup)
					*pPUCR |= pi.bMask;
				else
					*pPUCR &= ~(pi.bMask);
			}
			
		}
	}	
			
}

//=============================================================================
// output(u8 _bpin)
//=============================================================================
void output (u8 _bpin)
{
	// Lets use our table to do the work for us...
	if (_bpin < (sizeof(g_PortTable)/sizeof(PORTITEM))) {
		// Ok lets get the port table item...
		PORTITEM pi = g_PortTable[_bpin];
		
		// Make sure it is a valid one...
		if (pi.pPDR) {
			// See if there is a PMR register associated with this pin.
			if ((pi.bPU_PM) & 0xf) {
				volatile register unsigned char *pPMR = &IO.PMR1.BYTE + (pi.bPU_PM & 0xf) - 1;	// get the right PMR register
				*pPMR &= ~(pi.bMask);
			}
			
			// Next lets update PDR/PCR...
			u8 bIOPortIndex = (pi.pPDR - &IO.PDR1.BYTE);
			volatile unsigned char *pPCR = &IO.PCR1 + bIOPortIndex;
			bap_bpcrshadowregs[bIOPortIndex] |= pi.bMask;				// Ok update shadaw mask to say it is output
			*pPCR = bap_bpcrshadowregs[bIOPortIndex];
		}
	}	
			

}

//=============================================================================
// pinMode (u8 _bpin, u8 _bmode);	// some compatibility with Arduino command set.
//=============================================================================
void pinMode (u8 _bpin, u8 _bmode)	// some compatibility with Arduino command set.
{
	if (_bmode == INPUT)
		input(_bpin);
	if (_bmode == OUTPUT)
		output(_bpin);
}

//=============================================================================
// setpin - Set the pin to high or low
//=============================================================================
void setpin (u8 _bpin, u8 _bstate)
{
	if (_bstate == _LOW_)
		low (_bpin);
	else
		high (_bpin);
}

//=============================================================================
// low (u8 _bpin)
//=============================================================================
void low (u8 _bpin)
{
	// Lets use our table to do the work for us...
	if (_bpin < (sizeof(g_PortTable)/sizeof(PORTITEM))) {
		// Ok lets get the port table item...
		PORTITEM pi = g_PortTable[_bpin];
		
		// Make sure it is a valid one...
		if (pi.pPDR)
			PI_LOW(pi);
	}
}

//=============================================================================
// high (u8 _bpin)
//=============================================================================
void high (u8 _bpin)
{
	// Lets use our table to do the work for us...
	if (_bpin < (sizeof(g_PortTable)/sizeof(PORTITEM))) {
		// Ok lets get the port table item...
		PORTITEM pi = g_PortTable[_bpin];
		
		// Make sure it is a valid one...
		if (pi.pPDR)
			PI_HIGH(pi);
	}
}

// set the pin high
//=============================================================================
// Toggle (u8 _bpin)
//=============================================================================
void toggle (u8 _bpin)
{
	// Lets use our table to do the work for us...
	if (_bpin < (sizeof(g_PortTable)/sizeof(PORTITEM))) {
		// Ok lets get the port table item...
		PORTITEM pi = g_PortTable[_bpin];
		
		// Make sure it is a valid one...
		if (pi.pPDR)
			PI_TOGGLE(pi);
	}
}


//=============================================================================
// u8 in (u8 _bpin)
//=============================================================================
u8 in (u8 _bpin)
{
	// Lets use our table to do the work for us...
	if (_bpin < (sizeof(g_PortTable)/sizeof(PORTITEM))) {
		// Ok lets get the port table item...
		PORTITEM pi = g_PortTable[_bpin];
		
		// Make sure it is a valid one...
		if (pi.pPDR)
			return PI_ISHIGH(pi);
	}
	return 0;	
}

//=============================================================================
// u16 adin(u8 _bpin)
//=============================================================================
// analog-to-digital conversion
u16 adin (u8 _bpin)
{
	u16 wadc = 0;

	AD.ADCSR.BIT.ADST = 0; // stop conversion
	AD.ADCSR.BIT.ADIE = _FALSE_; // disable interrupt
	AD.ADCSR.BIT.SCAN = _BA_ADC_MODE_SINGLE_;
	AD.ADCSR.BIT.CKS = _BA_ADC_CLOCK_134_;
	AD.ADCR.BIT.TRGE = _FALSE_; // no external trigger
	switch (_bpin) 	{
		case 0:
		case 1:
		case 2:
		case 3:
			AD.ADCSR.BIT.CH = _bpin;
			break;
		case 16:
			AD.ADCSR.BIT.CH = 7;
			break;
		case 17:
			AD.ADCSR.BIT.CH = 6;
			break;
		case 18:
			AD.ADCSR.BIT.CH = 5;
			break;
		case 19:
			AD.ADCSR.BIT.CH = 4;
			break;
	}
	AD.ADCSR.BIT.ADST = 1; // start conversion
	while (AD.ADCSR.BIT.ADF != 1);
	while (AD.ADCSR.BIT.ADST == 1);
	switch (_bpin) {
		case 0:
		case 19:
			wadc = AD.ADDRA;
			break;
		case 1:
		case 18:
			wadc = AD.ADDRB;
			break;
		case 2:
		case 17:
			wadc = AD.ADDRC;
			break;
		case 3:
		case 16:
			wadc = AD.ADDRD;
			break;
	}
	return (wadc>>6);
}

//=============================================================================
// MultiAdin(u8 badcsr, u16 *pawatds) - Do multiple AtoD conversions. - Not
// 		very abstraced from the hardware, but... pass in the flags to use
//		to start up the AtoDs and then cycle.  We will have at least three
//		cases:
//			1) Startup the scan... specify which channels to use
//			2) Do next set of samples... 
//			3) Shut it down.
//		There may be other cases where we wish to change channels/config, but
//		for now punt (close/restart...)
//=============================================================================
// analog-to-digital conversion
void MultiAdin (u8 badcsr, u16 *pawatds)
{

	if (!pawatds) {
		// Start with easy case.  if pawatds = NULL then stop conversion.
		AD.ADCSR.BYTE = 0; 	// stop the conversion clear out the other fields while I am at it.
									// that way the IO pins are freed up as well...
									
	} else {
		if (!AD.ADCSR.BIT.ADST)	{
			// Was not already started, so lets initialize things...
			AD.ADCSR.BIT.ADF = 0;	       				// Clear the finished a pass flag...
            AD.ADCSR.BIT.ADIE = 0;						// We do not want interrupts
            AD.ADCSR.BIT.SCAN = 1;						// Yes lets start a scan.
            AD.ADCSR.BIT.CKS = (badcsr & 0x8)? 1 : 0;	// set clock speed
            AD.ADCSR.BIT.CH =  badcsr &0x7; 			// Setup which analog channels we want to start

			// now startup the scan                    
            AD.ADCSR.BIT.ADST = 1;       
		}
		
		// Now lets wait for a pass to complete...
		while (AD.ADCSR.BIT.ADF == 0) ;
		
		// Now lets return the Analog conversion data from the registers.
		*pawatds++ = AD.ADDRA >> 6;
		if  (badcsr & 0x3) {
			*pawatds++ = AD.ADDRB >> 6;
			if  (badcsr & 0x2) {
				*pawatds++ = AD.ADDRC >> 6;
				if  (badcsr & 0x1)
					*pawatds++ = AD.ADDRD >> 6;
			}
		}
		AD.ADCSR.BIT.ADF = 0;	       				// so we know if another pass has completed or not
	}
}


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//==============================================================================
// [Preliminary sound stuff] - move to approriate place...
//==============================================================================
// BUGBUG:: move to somewhere...
void Sound (u8 bPin, unsigned long wDur, unsigned long wFreq)			// dur in mS and Freq in Herz.  or should it be like basic???
{
	PORTITEM	piPin;									// The port item for the specified pin
	unsigned long	cCycles;								// the number of cycles to make this sound
	register unsigned long cPerHalfCycle;				// how many times to loop through per half cycle
	register unsigned long i;							// counter to use for the loops
	piPin = g_PortTable[bPin];							// get the port table item for the specified pin...
	PI_LOW(piPin);
	output(bPin);										// set the pin to output.
	
	// first calculate how many cycles of the tone we will produce
	cPerHalfCycle = (unsigned long)16000000L/(unsigned long)(wFreq * 2 *8)-3;			// *2(halves) *8(2dec, 2mov, 4bne) guess per count  - guess for setup and high/low
	cCycles = ((unsigned long)wDur * wFreq)/1000;

	while (cCycles--) {
		i = cPerHalfCycle;
		while (--i)	;
		PI_HIGH(piPin);
		i = cPerHalfCycle;
		while (--i)	;
		PI_LOW(piPin);										// leave low
	} 	

}

void MSound(u8 bPin, u8 cNotes, ...)
{
	va_list ap;
	word uDur;
	word uFreq;
	va_start(ap, cNotes);

	while (cNotes > 0) {
		uDur = va_arg(ap, unsigned int);
		uFreq = va_arg(ap, unsigned int);
		Sound(bPin, uDur, uFreq);
		cNotes--;
	}
	va_end(ap);
}

#ifdef __cplusplus
}
#endif
