/****************************************************************************
* BapSupport.c - Other support functions that need a home...
*
****************************************************************************/

#include "BapLib.h"

#ifdef __cplusplus
extern "C" {
#endif

unsigned long 	lTimerCnt;   //used in WTimer overflow. Will keep a 16 bit overflow so we have a 32 bit timer 

//--------------------------------------------------------------------
//[GetCurrentTime] - Gets the Timer value from our overflow counter as well as the TCA counter.  It
//                makes sure of consistancy. That is it is very posible that 
//                after we grabed the timers value it overflows, before we grab the other part
//                so we check to make sure it is correct and if necesary regrab things.
//--------------------------------------------------------------------
	
unsigned long GetCurrentTime(void)
{
	unsigned long lCurrentTime;
	lCurrentTime = lTimerCnt + TA.TCA;
	
	// handle wrap
	if (lTimerCnt != (lCurrentTime & 0xffffff00))
		lCurrentTime = lTimerCnt + TA.TCA;

	return lCurrentTime;
}

extern "C" void TIMERAV() __attribute__ ((interrupt_handler));
extern "C" void TIMERAV()
{
	toggle(P6);			
	lTimerCnt += 256;		// increment our counter TCA is the low order byte so increment past it...
	IRR1.BIT.IRRTA = 0;		// clear out the overflow flag
}

long iabs(long s)
{
	if (s >= 0)
		return s;
	return -s;
}

long imin (long a, long b)
{
	if (a <= b)
		return a;
	return b;	
}

long imax (long a, long b)
{
	if (a >= b)
		return a;
	return b;	
}


//--------------------------------------------------------------------
// [enable] - Enable specific interrupts or master switch
//--------------------------------------------------------------------
void enable(INTS ints)
{
	switch (ints){
		case _MASTER_INT_:
			and_ccr(0x7f);					// should clear the interrupt disable mask
			break;

		case TIMERAINT:
			IENR1.BIT.IENTA = 1; 		// ENABLE TIMERAINT 
			break;
	}
}

//--------------------------------------------------------------------
// [disable] - Disable specific interrupts - or master switch
//--------------------------------------------------------------------
void disable(INTS ints)
{
	switch (ints)
	{
		case _MASTER_INT_:
			or_ccr(0x80);				// This should turn on the Int flag which will mask interrupts)
			break;

		case TIMERAINT:
			IENR1.BIT.IENTA = 0; 		// DISABLE TIMERAINT 
			break;
	}
}

#ifdef __cplusplus
}
#endif
