//Project Lynxmotion Phoenix
//Description: Phoenix, control file.
//		The control input subroutine for the phoenix software is placed in this file.
//		Can be used with V2.0 and above
//Configuration version: V1.0
//Date: 25-10-2009
//Programmer: Jeroen Janssen (aka Xan)
//
//Hardware setup: PS2 version
// 
//NEW IN V1.0
//	- First Release
//
//	Walk method 1:
//	- Left Stick	Walk/Strafe
//	- Right Stick	Rotate
//
//	Walk method 2:
//	- Left Stick	Disable
//	- Right Stick	Walk/Rotate
//
//
//PS2 CONTROLS:
//	[Common Controls]
//	- Start			Turn on/off the bot
//	- L1			Toggle Shift mode
//	- L2			Toggle Rotate mode
//	- Circle		Toggle Single leg mode
//   - Square        Toggle Balance mode
//	- Triangle		Move body to 35 mm from the ground (walk pos) 
//					and back to the ground
//	- D-Pad up		Body up 10 mm
//	- D-Pad down	Body down 10 mm
//	- D-Pad left	decrease speed with 50mS
//	- D-Pad right	increase speed with 50mS
//
//	[Walk Controls]
//	- select		Switch gaits
//	- Left Stick	(Walk mode 1) Walk/Strafe
//				 	(Walk mode 2) Disable
//	- Right Stick	(Walk mode 1) Rotate, 		
//					(Walk mode 2) Walk/Rotate
//	- R1			Toggle Double gait travel speed
//	- R2			Toggle Double gait travel length
//
//	[Shift Controls]
//	- Left Stick	Shift body X/Z
//	- Right Stick	Shift body Y and rotate body Y
//
//	[Rotate Controls]
//	- Left Stick	Rotate body X/Z
//	- Right Stick	Rotate body Y	
//
//	[Single leg Controls]
//	- select		Switch legs
//	- Left Stick	Move Leg X/Z (relative)
//	- Right Stick	Move Leg Y (absolute)
//	- R2			Hold/release leg position
//
//	[GP Player Controls]
//	- select		Switch Sequences
//	- R2			Start Sequence
//
//====================================================================
// [Include files]
#include "globals.h"
#ifdef USEPS2

#ifdef __cplusplus
extern "C" {
#endif

//[CONSTANTS]
#define WALKMODE		0
#define TRANSLATEMODE	1
#define ROTATEMODE		2
#define SINGLELEGMODE	3
#define GPPLAYERMODE	4

#define PS2_DAT		P0
#define PS2_CMD		P1
#define PS2_SEL		P2
#define PS2_CLK		P3
#define cTravelDeadZone	 4		//The deadzone for the analog input from the remote

//-----------------------------------------------------------------------------

// Main data packet						
typedef struct _ps2packet {
	union
	{
		byte	bData[6];
		struct
		{
			byte fNLeft : 1; 
			byte fNDown : 1;  
			byte fNRight: 1;  
			byte fNUp : 1; 
			byte fNStart : 1;
			byte fNR3 : 1; 				// is pressed if the value is 0...
			byte fNL3 : 1;				// Warning these are Negaitve ie button
			byte fNSelect :1;			// These are the 16 buttons...

			byte fNSquare : 1;
			byte fNX : 1; 
			byte fNO : 1; 
			byte fNTriangle : 1; 
			byte fNR1 : 1; 
			byte fNL1 : 1; 
			byte fNR2 : 1; 
			byte fNL2 : 1; 

			byte	bRJoyLR;					// right joystick X (LR)
			byte	bRJoyUD;					//				..Y (UD)
			byte	bLJoyLR;					// Left Joystick  X (LR)
			byte	bLJoyUD;					// 				  Y (UD)
		} s;
	};
} PS2PACKET;

//=============================================================================
// Global - Local to this file only...
//=============================================================================

PS2PACKET 			g_ps2p;
PS2PACKET 			g_ps2pPrev;

static unsigned			g_BodyYOffset; 
static char				g_BodyYSift;	
static byte				ControlMode;
static bool				DoubleHeightOn;
static bool				DoubleTravelOn;
static bool				WalkMethod;		

PORTITEM			g_piCMD;
PORTITEM			g_piDAT;
PORTITEM			g_piSEL;
PORTITEM			g_piCLK;

//-----------------------------------------------------------------------------
// Main PS2 input functions
//-----------------------------------------------------------------------------
void PS2_Init(void) 
{
	DBGPrintf("PS2 Init\r");
   	input(PS2_DAT, true);  				// make the dat line input But also enable pull-up
	output(PS2_CMD);				// Make command pin be output
	output(PS2_SEL);				// init the select pin
	output(PS2_CLK);				// Make clock be output

	// First pass see if using our array of Porttable items with constants will generate same code as local copy...
	g_piCMD = g_PortTable[PS2_CMD];		
	g_piDAT = g_PortTable[PS2_DAT];
	g_piSEL = g_PortTable[PS2_SEL];
	g_piCLK = g_PortTable[PS2_CLK];

	PI_HIGH(g_piSEL);					// Initialize to high...
};

// PSx controller communication function.
// send a byte on the command line and receive one on the data line.
// needs Attention pin to have gone low before called to activate controller.
byte gameByte(register byte command)
{
        short int i ;
		short int j;
        // pause(1);
        register short int data = 0x00;                                   // clear data variable to save setting low bits later.
        for(i=0;i<8;i++)
        {
			if (command & 0x1)
				PI_HIGH(g_piCMD);		//sbi(PORTK, PScommand);      // bit bang "command" out on PScommand wire.
            else 
				PI_LOW(g_piCMD);		//cbi(PORTK, PScommand);
            PI_LOW(g_piCLK);			//cbi(PORTK, PSclock);                             // CLOCK LOW
			for (j=5; j > 0; j--)
				;
            if ( PI_ISHIGH(g_piDAT))		//if((PINK & _BV(PSdata))) 
				data |= (1<<i);		// sbi(data, i); 
            PI_HIGH(g_piCLK);			//sbi(PORTK, PSclock);                             // CLOCK HIGH
			command = command >> 1;
        }
        PI_HIGH(g_piCMD);				//sbi(PORTK, PScommand);

//        delay_us(20);                                            // wait for ACK to pass.
		for (j=20; j > 0; j--)
				;
		

        return(data);
}

void _PS2_ShiftOut(unsigned char *pb, int cb)
{
	while (cb)
	{
		gameByte(*pb++);
		cb--;
	}
}

byte PS2_Query(unsigned char *paData, int cb)
{
	int iLoopCnt;
	unsigned char bMode;

	//pause(100);		// see if timing is keeping the two from talking	
	// Hack to not have this loop forever in case of error
	for (iLoopCnt = 0; iLoopCnt < 10; iLoopCnt++)
	{
		//spiDeviceSelect(PS2_device, 1);
		PI_LOW(g_piSEL);
		
		
//		shiftout CMD,CLK,FASTLSBPRE,[$1\8]
		gameByte(0x1);
		
		//shiftin DAT,CLK,FASTLSBPOST,[DS2Mode\8]
		bMode = gameByte(0x42);

		if ((bMode & 0xf0) == 0x70)
		{
			// we are in an analog mode so continue to read in the extra bytes.
			gameByte(0);		// ignore first byte which should return 0x5a
			while (cb > 0)
			{
				*paData++ = gameByte(0);
				cb--;
			}

			// And clear our select line...
			PI_HIGH(g_piSEL);
		
			return 1;
		}

		//DBGPrintf("PS2 Query - Not Analog %x \r", bMode);
		PI_HIGH(g_piSEL);
		pause(1);

		// Not initialized properly, try to send out the correct init sequences.
		PI_LOW(g_piSEL);
//		shiftout CMD,CLK,FASTLSBPRE,[$1\8,$43\8,$0\8,$1\8,$0\8] ;CONFIG_MODE_ENTER
		static unsigned char _aps2_2a[] = {0x01, 0x43, 0x0, 0x1, 0x0};
		_PS2_ShiftOut(_aps2_2a, sizeof(_aps2_2a));
		PI_HIGH(g_piSEL);
		pause(1);
	
		PI_LOW(g_piSEL);
//		shiftout CMD,CLK,FASTLSBPRE,[$01\8,$44\8,$00\8,$01\8,$03\8,$00\8,$00\8,$00\8,$00\8] ;SET_MODE_AND_LOCK
		static unsigned char _aps2_3[] = {0x01, 0x44, 0, 1, 3, 0, 0, 0, 0};
		_PS2_ShiftOut(_aps2_3, sizeof(_aps2_3));
		PI_HIGH(g_piSEL);
		pause(1);
	
		//;pin_low(g_piSEL);
		//;shiftout CMD,CLK,FASTLSBPRE,[$01\8,$4D\8,$00\8,$00\8,$01\8,$FF\8,$FF\8,$FF\8,$FF\8] ;VIBRATION_ENABLE
		//;pin_high(g_piSEL);
		//;clockWaitms(1);

		//pin_low(g_piSEL);
        //shiftout CMD,CLK,FASTLSBPRE,[$01\8,$4F\8,$00\8,$FF\8,$FF\8,$03\8,$00\8,$00\8,$00\8] ;SET_DS2_NATIVE_MODE
		//static unsigned char _aps2_4[] = {0x01, 0x4F, 0, 0xff, 0xff, 3, 0, 0, 0};
		//_PS2_ShiftOut(_aps2_4, sizeof(_aps2_4));
		//pin_high(g_piSEL);
		//clockWaitms(1);
	
		PI_LOW(g_piSEL);
//		shiftout CMD,CLK,FASTLSBPRE,[$01\8,$43\8,$00\8,$00\8,$5A\8,$5A\8,$5A\8,$5A\8,$5A\8] ;CONFIG_MODE_EXIT_DS2_NATIVE
		static unsigned char _aps2_5[] = {0x01, 0x43, 0, 0, 0x5a, 0x5a, 0x5a, 0x5a, 0x5a};
		_PS2_ShiftOut(_aps2_5, sizeof(_aps2_5));
		PI_HIGH(g_piSEL);
//		clockWaitms(1);
	
//		pin_low(g_piSEL);
//		shiftout CMD,CLK,FASTLSBPRE,[$01\8,$43\8,$00\8,$00\8,$00\8,$00\8,$00\8,$00\8,$00\8] ;CONFIG_MODE_EXIT
////		static unsigned char _aps2_6[] = {0x01, 0x43, 0, 0, 0, 0, 0, 0, 0};
//		_PS2_ShiftOut(_aps2_6, sizeof(_aps2_6));
//		pin_high(g_piSEL);
		pause(144); //nap 4
	}
	
	// If we got here it means that we tried a number of times and it failed, so set up a set of default values and return.
	
	paData[0] = 255;
	paData[1] = 255; 
	paData[2] = 128;
	paData[3] = 128;
	paData[4] = 128;
	paData[5] = 128;
	
	return 0;
}

//==============================================================================
// This is The function that is called by the Main program to initialize
//			the input controller, which in this case is the PS2 controller
//			process any commands.
//==============================================================================

// If both PS2 and XBee are defined then we will become secondary to the xbee
#ifdef USEXBEE
void InitPS2Controller(void)
#else
void InitController(void)
#endif
{
  	g_BodyYOffset = 0;
  	g_BodyYSift = 0;
	PS2_Init();
}

//==============================================================================
// This function is called by the main code to tell us when it is about to
// do a lot of bit-bang outputs and it would like us to minimize any interrupts
// that we do while it is active...
//==============================================================================
#ifndef USEXBEE		// XBEE will provide this if defined...
void AllowControllerInterrupts(bool fAllow)
{
	// We don't need to do anything...
}
#endif

//==============================================================================
// This is The main code to input function to read inputs from the PS2 and then
//			process any commands.
//==============================================================================
#ifdef USEXBEE
void PS2ControlInput(void)
#else
void ControlInput(void)
#endif
{
	bool fPacketChanged;
		
	// Then try to receive a packet of information from the PS2.
	
	if (PS2_Query(g_ps2p.bData, sizeof(g_ps2p)))
	{
		fPacketChanged = !FCompareMem((const char *)(void*)&g_ps2p.bData, (const char *)(void*)&g_ps2pPrev.bData, sizeof(g_ps2p));

#ifdef DEBUG_PS2
		if (fPacketChanged) 
		{
			DBGPrintf("%2x%2x %2x %2x %2x %2x\r",g_ps2p.bData[0],g_ps2p.bData[1], 
						g_ps2p.s.bRJoyLR, g_ps2p.s.bRJoyUD, g_ps2p.s.bLJoyLR, g_ps2p.s.bLJoyUD);
		}
#endif
		// OK lets try "0" button for Start. 
		if (!g_ps2p.s.fNStart && g_ps2pPrev.s.fNStart)
		{	//Start Button (0 on keypad) test
			DBGPrintf("Start button Pressed\r");
			if(g_fHexOn)
			{
				//Turn off
	  			BodyPosX = 0;
	  			BodyPosY = 0;
	  			BodyPosZ = 0;
	  			BodyRotX1 = 0;
	  			BodyRotY1 = 0;
	  			BodyRotZ1 = 0;
	  			TravelLengthX = 0;
	  			TravelLengthZ = 0;
	  			TravelRotationY = 0;
	  			g_BodyYOffset = 0;
	  			g_BodyYSift = 0;
	  			SelectedLeg = 255;
			
				g_fHexOn = 0;
			}	
			else
			{
				//Turn on
				g_fHexOn = 1;
			}
		}	
		
		if (g_fHexOn)
		{
			// [SWITCH MODES]
    
			//Translate mode
			if (!g_ps2p.s.fNL1 && g_ps2pPrev.s.fNL1)	// L1 Button Test
			{
				Sound(P9, 50, 2000);  	//sound P9, [50\4000]
	  			if (ControlMode != TRANSLATEMODE )
	    			ControlMode = TRANSLATEMODE;
	  			else
	  			{
	    			if (SelectedLeg==255) 
	      				ControlMode = WALKMODE;
	    			else
	      				ControlMode = SINGLELEGMODE;
				}
			}
  	
	    	//Rotate mode
			if (!g_ps2p.s.fNL2 && g_ps2pPrev.s.fNL2)	// L2 Button Test
			{
				Sound(P9, 50, 2000);  	//sound P9, [50\4000]
	  			if (ControlMode != ROTATEMODE)
	    			ControlMode = ROTATEMODE;
	  			else
	  			{
	    			if (SelectedLeg == 255) 
	      				ControlMode = WALKMODE;
	    			else
	      				ControlMode = SINGLELEGMODE;
	    		}
			}
	    	
	    	//Single leg mode fNO
			if (!g_ps2p.s.fNO && g_ps2pPrev.s.fNO)	// O - Circle Button Test
			{
	  			if (iabs(TravelLengthX)<cTravelDeadZone && iabs(TravelLengthZ)<cTravelDeadZone && iabs(TravelRotationY*2)<cTravelDeadZone )
	  			{
	    			//Sound P9,[50\4000]
	    			if (ControlMode != SINGLELEGMODE) 
	    			{
	      				ControlMode = SINGLELEGMODE;
	      				if (SelectedLeg == 255)  //Select leg if none is selected
	        				SelectedLeg=cRF;	 //Startleg
	        		}
	  				else
	  				{
						ControlMode = WALKMODE;
	      				SelectedLeg=255	;
	      			}
	      		}
			}	      	
	
	    	// GP Player Mode X
			if (!g_ps2p.s.fNX && g_ps2pPrev.s.fNX)	// X - Circle Button Test
			{
				Sound(P9, 50, 2000);  	//sound P9, [50\4000]
	  			if (ControlMode != GPPLAYERMODE)
	  			{
	    			ControlMode = GPPLAYERMODE;
	    			GPSeq=0;
	    		}
	  			else
	    			ControlMode = WALKMODE;
	    	}
	
	//[Common functions]
			//Switch Balance mode on/off 
			if (!g_ps2p.s.fNSquare && g_ps2pPrev.s.fNSquare)	// Square Button Test
			{
	  			BalanceMode = !BalanceMode;
	  			if (BalanceMode )
	  			{
					Sound(P9, 250, 1500);  	//sound P9, [250\3000]
				}
	  			else	  
	  			{
	  				MSound(9, 2, 100, 2000, 50, 4000);
					//sound P9,[100\4000, 50\8000]
				}
			}
	
			//Stand up, sit down  
			if (!g_ps2p.s.fNTriangle && g_ps2pPrev.s.fNTriangle)	// Triangle - Button Test
			{
	  			if (g_BodyYOffset>0) 
	    			g_BodyYOffset = 0;
	  			else
	    			g_BodyYOffset = 35;
	    	}
	
			if (!g_ps2p.s.fNUp && g_ps2pPrev.s.fNUp)			// D-Up - Button Test
	  			g_BodyYOffset = g_BodyYOffset+10;
			
			if (!g_ps2p.s.fNDown && g_ps2pPrev.s.fNDown)		// D-Down - Button Test
				g_BodyYOffset = g_BodyYOffset-10;
		
			if (!g_ps2p.s.fNRight && g_ps2pPrev.s.fNRight)	// D-Right - Button Test
			{
	  			if (SpeedControl>0 )
	  			{
	    			SpeedControl = SpeedControl - 50;
					Sound(P9, 50, 2000);  	//sound P9, [50\4000]
	    		}
	    	}	
	
			if (!g_ps2p.s.fNLeft && g_ps2pPrev.s.fNLeft)	// D-Left - Button Test
			{			
	  			if (SpeedControl<2000 )
	  			{
	    			SpeedControl = SpeedControl + 50;
					Sound(P9, 50, 2000);  	//sound P9, [50\4000]
	    		}
	    	}
	
			//[Walk functions]
			if (ControlMode == WALKMODE) 
			{
		
	  			//Switch gates
				if ((!g_ps2p.s.fNSelect && g_ps2pPrev.s.fNSelect)	// Select Button Test
						&& iabs(TravelLengthX)<cTravelDeadZone 		//No movement
						&& iabs(TravelLengthZ)<cTravelDeadZone 
						&& iabs(TravelRotationY*2)<cTravelDeadZone  )
				{
  					if (GaitType<7)
  					{
						Sound(P9, 50, 2000);  	//sound P9, [50\4000]
		  				GaitType = GaitType+1;
		  			}
  	  				else
  	  				{
					  	MSound (9, 2, 50, 2000, 50, 2250); 
					  	
		  				GaitType = 0;
		  			}
		  			GaitSelect();
		  		}
	  		
	  			//Double leg lift height		
				if (!g_ps2p.s.fNR1 && g_ps2pPrev.s.fNR1)	// R1 Button Test
				{
					Sound(P9, 50, 2000);  	//sound P9, [50\4000]
	    			DoubleHeightOn = !DoubleHeightOn;
			    	if (DoubleHeightOn)
	      				LegLiftHeight = 80;
	    			else
	  	  				LegLiftHeight = 50;
	  	  		}
	  		
		  		//Double Travel Length
				if (!g_ps2p.s.fNR2 && g_ps2pPrev.s.fNR2)	// R2 Button Test
				{
					Sound (P9, 50, 2000);  	//sound P9, [50\4000]
	    			DoubleTravelOn = !DoubleTravelOn;
	    		}
	  	
		  		// Switch between Walk method 1 && Walk method 2
				if (!g_ps2p.s.fNR3 && g_ps2pPrev.s.fNR3)	// R3 Button Test
				{
					Sound (P9, 50, 2000);  	//sound P9, [50\4000]
	    			WalkMethod = !WalkMethod;
	    		}
	  		
				//Walking	
				if (WalkMethod)  //(Walk Methode) 
					TravelLengthZ = (g_ps2p.s.bRJoyUD-128); //Right Stick Up/Down	  
		
				else	  
				{
					TravelLengthX = -(g_ps2p.s.bLJoyLR - 128);
					TravelLengthZ = (g_ps2p.s.bLJoyUD - 128);
				}
							
				if (!DoubleTravelOn)  //(Double travel length)
				{
					TravelLengthX = TravelLengthX/2;
					TravelLengthZ = TravelLengthZ/2;
				}
				
				TravelRotationY = -(g_ps2p.s.bRJoyLR - 128)/4;	 //Right Stick Left/Right 
			}	
		
			//[Translate functions]	
			//g_BodyYSift = 0	
			if (ControlMode == TRANSLATEMODE) 	
			{
	  			BodyPosX = (g_ps2p.s.bLJoyLR - 128)/2;
	  			BodyPosZ = -(g_ps2p.s.bLJoyUD - 128)/3;
	  			BodyRotY1 = (g_ps2p.s.bRJoyLR - 128)*2;
	  			g_BodyYSift = (-(g_ps2p.s.bRJoyUD - 128)/2);
			}
		
			//[Rotate functions]	
			if (ControlMode == ROTATEMODE) 
			{
				BodyRotX1 = (g_ps2p.s.bLJoyUD - 128);
				BodyRotY1 = (g_ps2p.s.bRJoyLR - 128)*2;
				BodyRotZ1 = (g_ps2p.s.bLJoyLR - 128);
				g_BodyYSift = (-(g_ps2p.s.bRJoyUD - 128)/2);
			}
		
			//[Single leg functions]	
			if (ControlMode == SINGLELEGMODE) 
			{
				//Switch leg for single leg control
				if (!g_ps2p.s.fNSelect && g_ps2pPrev.s.fNSelect)	// Select Button Test
				{
					Sound (P9, 50, 2000);  	//sound P9, [50\4000]
					if (SelectedLeg<5)
						SelectedLeg = SelectedLeg+1;
					else
						SelectedLeg=0;
				}
	
	    		SLLegX	= (g_ps2p.s.bLJoyLR - 128)/2; 		//Left Stick Right/Left
	    		SLLegY	= (g_ps2p.s.bRJoyUD - 128)/10; 		//Right Stick Up/Down
	    		SLLegZ 	= (g_ps2p.s.bLJoyUD - 128)/2; 		//Left Stick Up/Down
		
		  		// Hold single leg in place
				if (!g_ps2p.s.fNR2 && g_ps2pPrev.s.fNR2)	// R2 Button Test
				{
					Sound (P9, 50, 2000);  	//sound P9, [50\4000]
	    			fSLHold = !fSLHold;
	    		}
			}
  		
	  		//[GPPlayer functions]	
			if (ControlMode == GPPLAYERMODE) 
			{
		
	  			//Switch between sequences
				if (!g_ps2p.s.fNSelect && g_ps2pPrev.s.fNSelect)	// Select Button Test
				{
	    			if (GPStart==0 )
	    			{
	      				if (GPSeq < 5)  //Max sequence
	      				{
							Sound (P9, 50, 1500);  	//sound P9, [50\3000]
	        				GPSeq = GPSeq+1;
	        			}
	      				else
	      				{
	        				MSound (P9, 2, 50, 2000, 50, 2250);	//Sound P9,[50\4000, 50\4500]
	        				GPSeq=0;
	        			}
	        		}
	        	}
				//Start Sequence
				if (!g_ps2p.s.fNR2 && g_ps2pPrev.s.fNR2)	// R2 Button Test
					GPStart=1;
			}
		
			//Calculate walking time delay
			InputTimeDelay = 128 - imax(imax(iabs(g_ps2p.s.bLJoyLR - 128), iabs(g_ps2p.s.bLJoyUD - 128)), iabs(g_ps2p.s.bRJoyLR - 128));
		}
  	
		//Calculate BodyPosY
		BodyPosY = imax(g_BodyYOffset + g_BodyYSift,  0);
	
		// Save away our last query
		g_ps2pPrev = g_ps2p;

	}
}

#ifdef __cplusplus
}
#endif


#endif //USEPS2

